function [price, lattice, latticeS] = BinomialExactQ(S0, K, r, T, sigma, M, put)

if nargin < 7
    put = 0;
end
dt = T/M; beta = 0.5 * ( exp(-r*dt) + exp((r+sigma^2)*dt) );
u = beta + sqrt( beta^2 -1 ); d=1/u; q = (exp(r*dt) - d) / (u-d);
lattice = zeros(M+1,M+1); latticeS = zeros(M+1,M+1);

for j = 0:M
    latticeS(j+1,M+1) = S0*u^j*d^(M-j);
    if (put)
        lattice(j+1,M+1) = max(0, K - latticeS(j+1,M+1)); % Put Payoff
    else
        lattice(j+1,M+1) = max(0, latticeS(j+1,M+1) - K); % Call Payoff
    end
end

for i=M-1:-1:0
    for j=0:i
        latticeS(j+1,i+1) = S0*u^j*d^(i-j);
        lattice(j+1,i+1) = exp(-r*dt) * ...
            (q * lattice(j+2,i+2) + (1-q) * lattice(j+1,i+2));
   end   
end
price = lattice(1,1);
