function [sol, price] = BlackScholes_CrankNicolson_pSOR(S0, K, r, D, sigma, T, ...
    f, alpha, beta, Smin, Smax, dS, dt, omega, tol)
% f, alpha, beta : functions of initial, (left and right, resp.) boundaries
% price : linearly interpolated

N = round((Smax-Smin)/dS); M = round(T/dt);
sol = zeros(N+1,M+1); S = Smin + dS*[0:N]; t = dt*[0:M]; 
sol(:,end) = feval(f, S, K); % t = T
sol(1,:)   = feval(alpha, t,T,r,K,Smin, D); % S = Smin
sol(end,:) = feval(beta, t,T,r,K,Smax, D); % S = Smax

mS = S/dS;
a = 0.25*dt*(mS.*(sigma^2*mS - (r-D))); 
b = -0.5*dt*(sigma^2*(mS.^2) + r);
c = 0.25*dt*(mS.*(sigma^2*mS + (r-D))); 
B =  diag(a(3:N),-1) + diag(1+b(2:N)) + diag(c(2:N-1),1); %[L U] = lu(A);

bj = zeros(N-1,1); P = feval(f, S(2:end-1), K)'; 
for j = M:-1:1 % for t
    bj(1) = a(2) * (sol(1,j) + sol(1,j+1));
    bj(end) = c(end-1) * (sol(end,j) + sol(end,j+1)); 
    yj = B*sol(2:N,j+1) + bj; 
    SOR = sol(2:end-1, j+1)'; error = realmax;  % start with a huge number
    while ( tol < error )
        AmSOR(1) = SOR(1) + omega/(1-b(2)) * ( yj(1) - ...
            (1-b(2))*SOR(1) + c(2)*SOR(2) );
        AmSOR(1) = max( AmSOR(1), P(1) );
        for i = 2:N-2
            AmSOR(i) = SOR(i) + omega/(1-b(i+1)) * ( yj(i) + ...
                a(i+1)*AmSOR(i-1) - (1-b(i+1))*SOR(i) + c(i+1)*SOR(i+1) );
            AmSOR(i) = max( AmSOR(i), P(i) );
        end
        AmSOR(N-1) = SOR(N-1) + omega/(1-b(N)) * ( yj(N-1) + ...
            a(N)*AmSOR(N-2) - (1-b(N))*SOR(N-1) );
        AmSOR(N-1) = max( AmSOR(N-1), P(N-1) );
        error = norm(AmSOR - SOR); SOR = AmSOR;
    end
    sol(:,j) = max([sol(1,j) AmSOR sol(end,j)], feval(f, S, K) ); 
end
down = floor((S0-Smin)/dS); up = ceil((S0-Smin)/dS);
if (down == up)
    price = sol(down+1,1);
else
    price = sol(down+1,1) + ...
        (sol(up+1,1) - sol(down+1,1))*(S0-Smin - down*dS)/dS;
end
