function [sol, price] = BlackScholes_Explicit(S0, K, r, D, sigma, T, ...
    f, alpha, beta, Smin, Smax, dS, dt)
% f, alpha, beta : functions of initial, (left and right, resp.) boundaries
% price : linearly interpolated

N = round((Smax-Smin)/dS); M = round(T/dt);
sol = zeros(N+1,M+1); S = Smin + dS*[0:N]; t = dt*[0:M]; 
% payoff and boundary from f, alpha, beta
sol(:,end) = feval(f, S,K); % t = T
sol(1,:)   = feval(alpha, t,T,r,K,Smin, D); % S = Smin
sol(end,:) = feval(beta, t,T,r,K,Smax, D); % S = Smax

mS = S/dS;
a = 0.5*dt*(mS.*(sigma^2*mS - (r-D)));
b = 1 - dt*( sigma^2*mS.^2 + r);
c = 0.5*dt*(mS.*(sigma^2*mS + (r-D)));
for j=M:-1:1    % for t
    for i=2:N % for S
        sol(i,j) = a(i)*sol(i-1,j+1) + ...
        b(i)*sol(i,j+1) + c(i)*sol(i+1,j+1);
    end
end
% linear interpolation
down = floor((S0-Smin)/dS); up = ceil((S0-Smin)/dS);
if (down == up)
    price = sol(down+1,1);
else
    price = sol(down+1,1) + ...
        (sol(up+1,1) - sol(down+1,1))*(S0-Smin - down*dS)/dS;
end