function [sol, price] = BlackScholes_Implicit(S0, K, r, D, sigma, T, ...
    f, alpha, beta, Smin, Smax, dS, dt)
% f, alpha, beta : functions of initial, (left and right, resp.) boundaries
% price : linearly interpolated

N = round((Smax-Smin)/dS); M = round(T/dt);
sol = zeros(N+1,M+1); S = Smin + dS*[0:N]; t = dt*[0:M]; 
sol(:,end) = feval(f, S,K); % t = T
sol(1,:)   = feval(alpha, t,T,r,K,Smin, D); % S = Smin
sol(end,:) = feval(beta, t,T,r,K,Smax, D); % S = Smax

mS = S/dS;
a = 0.5*dt*(mS.*((r-D) - sigma^2*mS));
b = 1 + dt*(sigma^2*mS.^2 + r);
c = -0.5*dt*(mS.*((r-D) + sigma^2*mS));
A = diag(a(3:N),-1) + diag(b(2:N)) + diag(c(2:N-1),1);
[L U] = lu(A);

bj = zeros(N-1,1);
for j = M:-1:1 % for t
    bj(1) = -a(2)*sol(1,j); bj(end) = -c(end-1)*sol(end,j); 
    sol(2:N,j) = U \ ( L \ (sol(2:N,j+1) + bj) );   
end
down = floor((S0-Smin)/dS); up = ceil((S0-Smin)/dS);
if (down == up)
    price = sol(down+1,1);
else
    price = sol(down+1,1) + ...
        (sol(up+1,1) - sol(down+1,1))*(S0-Smin - down*dS)/dS;
end