% Euler-Maruyama_Sqrt_MeanReverting (Vectorized across samples)
clear all, close all, randn('state',13)

K = 1; mu = 0.05; T = 1; N = 2^16; dt = T/N; M = 10;
S_0 = 1; Sig_0 = 0.8; Sigma0 = 0.8; S = S_0*ones(M,N); Sig = Sig_0*ones(M,N);
for j = 1:N
    dW_1 = sqrt(dt)*randn(M,1); dW_2 = sqrt(dt)*randn(M,1); % independent of dW_1
    % To be safe!
    S(:,j+1) = abs(S(:,j) + dt*mu*S(:,j) + sqrt(S(:,j)).*Sig(:,j).*dW_1);
    Sig(:,j+1) = abs(Sig(:,j) + dt*(Sigma0 - Sig(:,j)) + sqrt(Sig(:,j)).*dW_2);
end
figure(1), plot([0:dt:1], Sig(1,:), 'g', [0:dt:1], Sigma0*ones(1,size(Sig,2)), 'r' )
xlabel('time','FontSize',12), ylabel('volatility','FontSize',12),
print -r900 -depsc -cmyk '../figures/EM_tv'

figure(2), plot( [0:dt:1], S(1,:), 'b',[0:dt:1], 1+mu*[0:dt:1], 'r' )
xlabel('time','FontSize',12), ylabel('stock prices','FontSize',12)
print -r900 -depsc -cmyk '../figures/EM_ts'

figure(3), plot( S(1,:), Sig(1,:), 'r' ), grid on
xlabel('stock prices','FontSize',12), ylabel('volatility','FontSize',12)
print -r900 -depsc -cmyk '../figures/EM_sv'

figure(4), plot3( [0:dt:1], S(1,:), Sig(1,:), 'k' ), hold on
plot3( [0:dt:1], S(1,:), zeros(size(Sig(1,:))), 'b' )
plot3( ones(size([0:dt:1])), S(1,:), Sig(1,:), 'r' )
plot3( [0:dt:1], 3*ones(size(S(1,:))), Sig(1,:), 'g' ), grid on
xlabel('time','FontSize',12), ylabel('stock prices','FontSize',12)
zlabel('volatility','FontSize',12)
print -r900 -depsc -cmyk '../figures/EM_tsv'
