% Euler-Maruyama for Geometric Brownian Motion
% dS = mu*S*dt + sigma*S*dW, S_0 = 1
clear all, close all, randn('state',13)
T = 1; M = 2^8; dt = T/M; mu = 0.1; sigma = 0.5; S0 = 1; N = 10000;
HS = zeros(N,1); S = zeros(N,M+1); S(:,1) = S0*ones(1,N);
for k = 1:N % for a path 
    for j = 1:M
        dW = sqrt(dt)*randn;
        a = mu*S(k,j); b = sigma*S(k,j); % calculate coefficients
        S(k,j+1) = S(k,j) + a*dt + b*dW;
    end
end
figure(1), plot( [0:dt:T], S(1:1000:N, :) ), 
xlabel('t','FontSize',12), ylabel('S','FontSize',12)    
print -r900 -depsc -cmyk '../figures/GeometricBM_Paths'
figure(2), HS = S(:,end); 
histNormalized(HS, 0.05, min(min(HS)), max(max(HS))); hold on
x = (0:0.02:6); y = lognpdf(x, (mu - 0.5*sigma^2), sigma); % lognormal pdf
plot(x, y, 'r-', 'LineWidth', 2); xlabel('x','FontSize',12), ylabel('y','FontSize',12)
print -r900 -depsc -cmyk '../figures/GeometricBM'
