% Up-and-in call option: uses Monte Carlo with antithetic variates
clear all, close all, randn('state',13)

S = 5; E = 6; sigma = 0.3; r = 0.05; T = 1; B = 8; dt = 1e-4; N = T/dt; M = 1e3;
V = zeros(M,1); Vanti = zeros(M,1);
for i = 1:M
    samples = randn(N,1); 
    % standard Monte Carlo
    Svals = S*cumprod(exp((r-0.5*sigma^2)*dt+sigma*sqrt(dt)*samples));
    Smax = max(Svals);
    if Smax > B 
       V(i) = exp(-r*T) * max(Svals(end)-E,0);
    end
    % antithetic path 
    Svals2 = S*cumprod(exp((r-0.5*sigma^2)*dt-sigma*sqrt(dt)*samples));
    Smax2 = max(Svals2); V2 = 0;
    if Smax2 > B 
       V2 = exp(-r*T) * max(Svals2(end)-E,0);
    end
    Vanti(i) = 0.5*(V(i) + V2);
end
PriceV = mean(V); StdV = std(V);
CI = [PriceV - 1.96*StdV/sqrt(M), PriceV + 1.96*StdV/sqrt(M)]
PriceVanti = mean(Vanti); StdVanti = std(Vanti);
CIanti = [PriceVanti - 1.96*StdVanti/sqrt(M), ...
    PriceVanti + 1.96*StdVanti/sqrt(M)]
ratio = ( CI(2)-CI(1) ) / ( CIanti(2)-CIanti(1) )