% approximation of bond prices
clear all, close all
coupons = [10 10 10 10]; % given
yields = -0.1:0.01:0.6; howMany = length(yields);
for i = 1:howMany
    prices(i) = pvvar([0 coupons], yields(i));
end
    
plot(yields, prices) % exact prices
hold on
lambda0 = 0.05; P0 = pvvar([0 coupons], lambda0); % assumed to be given
[duration, modifiedDuration] = cfdur(coupons, lambda0); 
convexity = cfconv(coupons, lambda0); 

% first-order approximation
P_1 = P0 .* ( 1 - modifiedDuration .* (yields - lambda0) );
plot(yields, P_1, 'r--'), hold on
%second-order approximation
P_2 = P0 .* ( 1 - modifiedDuration .* (yields - lambda0)  ...
    + 0.5 * convexity .* (yields - lambda0).^2 );
plot(yields, P_2, 'g-.'), hold off
ylabel('bond prices','FontSize',12), xlabel('yields','FontSize',12)
legend('bond prices', 'first-order approx.', 'second-order approx.')
print -r900 -deps '../figures/approxBondPrices'
