function sol = heatCrankNicolson(f, alpha, beta, a, b, dx, dt, tmax)
% f, alpha, beta : functions of initial, (left and right, resp.) boundaries
% a, b : x \in [a,b]

N = round((b-a)/dx); M = round(tmax/dt); sol = zeros(N+1,M+1); 
lambda = dt / (dx)^2; dA = 2*(1+lambda); dB = 2*(1-lambda);
x = a + dx*[0:N]; t = dt*[0:M];

% initial and boundary from f, alpha, beta
sol(:,1) = feval(f, x); % t=0
sol(1,:)   = feval(alpha, t); % x = a
sol(end,:) = feval(beta, t); % x = b

A = diag((dA) * ones(N-1,1)) - ...
   diag(lambda*ones(N-2,1),1) - diag(lambda*ones(N-2,1),-1);
B = diag((dB) * ones(N-1,1)) + ...
   diag(lambda*ones(N-2,1),1) + diag(lambda*ones(N-2,1),-1);
[L, U] = lu(A);

bj = zeros(N-1,1);
for j=1:M
    bj(1) = lambda*(sol(1,j)+sol(1,j+1));
    bj(end) = lambda*(sol(end,j)+sol(end,j+1));
    sol(2:N,j+1) = U \ ( L \ ( B*sol(2:N,j) + bj ) );   
end
