function sol = heatExplicit(f, alpha, beta, a, b, dx, dt, tmax)
% f, alpha, beta : functions of initial, (left and right, resp.) boundaries
% a, b : x \in [a,b]

N = round((b-a)/dx); M = round(tmax/dt);
sol = zeros(N+1,M+1); lambda = dt / (dx)^2; dLambda = 1-2*lambda;
x = a + dx*[0:N]; t = dt*[0:M];

% initial and boundary from f, alpha, beta
sol(:,1) = feval(f, x); % t=0
sol(1,:)   = feval(alpha, t); % x = a
sol(end,:) = feval(beta, t); % x = b

% solution in the domain
for j=1:M    % for t
   for i=2:N % for x
      sol(i,j+1) = lambda*sol(i-1,j) + dLambda*sol(i,j) + lambda*sol(i+1,j);
   end
end
