function sol = heat_BlackScholes_CrankNicolson(f, leftB, rightB, ...
    a, b, dx, dt, tmax, gamma, beta)
% f, alpha, beta : functions of initial, (leftB and rightB, resp.) boundaries
% a, b : x \in [a,b]

N = round((b-a)/dx); M = round(tmax/dt); sol = zeros(N+1,M+1); 
lambda = dt / (dx)^2; dA = 2*(1+lambda); dB = 2*(1-lambda);
x = a + dx*[0:N]; t = dt*[0:M];
sol(:,1) = feval(f, x, gamma, beta); % t=0
sol(1,:)   = feval(leftB, t, gamma, beta, a); % x = a
sol(end,:) = feval(rightB, t, gamma, beta, b); % x = b

A = diag((dA) * ones(N-1,1)) - ...
   diag(lambda*ones(N-2,1),1) - diag(lambda*ones(N-2,1),-1);
B = diag((dB) * ones(N-1,1)) + ...
   diag(lambda*ones(N-2,1),1) + diag(lambda*ones(N-2,1),-1);
bj = zeros(N-1,1); [L, U] = lu(A);
for j=1:M
    bj(1) = lambda*(sol(1,j)+sol(1,j+1));
    bj(end) = lambda*(sol(end,j)+sol(end,j+1));
    sol(2:N,j+1) = U \ ( L \ ( B*sol(2:N,j) + bj ) );   
end
