function [Price, CI] = qMC_Halton_Call(S0,X,r,sigma,T,NPoints, Base1,Base2)
% Base1, Base2 are two bases (primes) for Halton Sequences

nuT = (r - 0.5*sigma^2)*T; siT = sigma * sqrt(T);
% Halton Sequences
H1 = getHaltonSequence(Base1, ceil(NPoints/2));
H2 = getHaltonSequence(Base2, ceil(NPoints/2));

% Use Box-Muller to generate standard normals
sqLog = sqrt(-2*log(H1));
Norm1 = sqLog .* cos(2*pi*H2); Norm2 = sqLog .* sin(2*pi*H2);
Norm = [Norm1 ; Norm2]; Norm = Norm(1:NPoints);

% Pricing 
DiscPayoff = exp(-r*T) * max( 0, S0*exp(nuT+siT*Norm) - X );
Price = mean(DiscPayoff); 
% assuming a statistical error
width = 1.96*std(DiscPayoff)/sqrt(NPoints);
CI = [Price-width, Price+width];
