% test Black-Scholes Solutions
clear all, close all
S0 = 50; K = 50; D = 0.1; sigma = 0.4; r = 0.1; Smin = 0; Smax = 200; 
T = 3; dS = 0.25; dt = 0.001; omega = 1.0; tol = 1e-8;

f = @BlackScholes_Payoff_Am; 
alpha = @BlackScholes_LeftBoundary_Am; beta = @BlackScholes_RightBoundary_Am;
[w,p] = BlackScholes_CrankNicolson_pSOR(S0, K, r, D, sigma, T, ...
     f, alpha, beta, Smin, Smax, dS, dt, omega, tol);

[n,m] = size(w); S = linspace(Smin,Smax,n); t = linspace(0,T,m);
Sf = zeros(1,m); DeltaW = zeros(size(w)); 
for j = 1:m-1
    difference = w(:,j) - feval(f, S, K)';
    tFirst = find(difference, 1,'first'); Sf(j) = S(tFirst);
end
DeltaW(1:end-1,:) = ( w(2:end,:) - w(1:(end-1),:) ) ./ dS;
mesh(S, t, DeltaW'), xlabel('S','FontSize',12), 
ylabel('t','FontSize',12), zlabel('\Delta(S,t)','FontSize',12), hold on, 
plot3([Sf((1:end-1)) K],t(1:end), zeros(size(w)), 'k-', 'LineWidth', 2),
print -r300 -depsc -cmyk '../figures/testAm_pSOR_Delta'
fprintf('PDE Solution.................: %f\n', p);
