% test Black-Scholes Solutions
clear all, close all, 
S0 = 50; K = 50; D = 0; sigma = 0.4; r = 0.1; T = 5/12;
Smin = 0; Smax = 150; % Smin = 40; for barrier option

dS = 2; dt = 1/1200; 
f = @BlackScholes_Payoff_Exp; 
alpha = @BlackScholes_LeftBoundary_Exp; beta = @BlackScholes_RightBoundary_Exp;
[w,p] = BlackScholes_Explicit(S0, K, r, D, sigma, T, ...
    f, alpha, beta, Smin, Smax, dS, dt);
% [w,p] = BlackScholes_Implicit(S0, K, r, D, sigma, T, ...
%    f, alpha, beta, Smin, Smax, dS, dt);
% [w,p] = BlackScholes_CrankNicolson(S0, K, r, D, sigma, T, ...
%     f, alpha, beta, Smin, Smax, dS, dt);

[m,n] = size(w); S = linspace(Smin,Smax,m); t = linspace(0,T,n);
% Closed-Form Solution
[call,cdelta,put,pdelta] = CallPut_Delta(S, K, r, sigma, T);

figure(1), plot(S, w(:,1), 'rx'), hold on, plot(S, call),
xlabel('S','FontSize', 12), ylabel('V(S,0)','FontSize', 12), 
legend('Explicit Method','Closed-Form'), hold off
print -r900 -depsc -cmyk '../figures/testBS_Explicit_2D'
figure(2), mesh(S,t,w'), xlabel('S','FontSize',12), ylabel('t','FontSize',12), 
zlabel('V(S,t)','FontSize',12)
print -r300 -depsc -cmyk '../figures/testBS_Explicit'

fprintf('PDE Solution...........: %f\n', p);
% Exact solution from a closed-form formula for S0
[cclosed,cdclosed,pclosed,pdclosed] = CallPut_Delta(S0, K, r, sigma, T);
fprintf('Closed-Form Solution...: %f\n', cclosed);
