% testHeat_BlackScholes
clear all, close all
S0 = 10; K = 10; r = 0.25; sigma = 0.6; div = 0; div = 0.2; T = 1; 
% Transformations
kappa = (r-div)/(sigma^2/2); ell = div/(sigma^2/2);
gamma = 0.5*(kappa - 1); beta = gamma+1; x0 = log(S0/K); tmax = 0.5*sigma^2*T;

xmin = -5; xmax = 5; dx = 0.05; dt = 0.00005; % dt = 0.01;
f = @heat_BlackScholes_InitialFunction; % but transformed from option data 
leftB = @heat_BlackScholes_LeftBoundary; rightB = @heat_BlackScholes_RightBoundary; 
w = heat_BlackScholes_CrankNicolson(f, leftB, rightB, ...
    xmin, xmax, dx, dt, tmax, gamma, beta);

[m,n] = size(w); x = linspace(xmin,xmax,m); tau = linspace(0,tmax,n);
S = K*exp(x); t = T - 2*tau/sigma^2; j = 1;
for time = tau % another defn of a "for" loop
    inExp = exp( -gamma*x' );
    inExp = inExp .* exp( -(beta^2 + ell)*time );
    V(:,j) = K * inExp .* w(:,j); j = j + 1;
end
[C, Cdelta, P, Pdelta] = CallPut_Delta(S,K,r,sigma,T,div); u = P; 
[C, Cdelta, P, Pdelta] = CallPut_Delta(S,K,r,sigma,0,div); payoff = P;
figure(1), plot(S, u), hold on % exact solution 
plot(S, V(:,end), 'rx-'), plot(S, payoff, 'k-.'); hold on 
axis([0 30 0 10]); legend('Closed-Form', 'PDE Solution', 'Payoff'); 
xlabel('S','FontSize',12), ylabel('V(:,end)','FontSize',12), hold off
print -r900 -depsc -cmyk '../figures/testHeat_BlackScholes_CrankNicolson_2D'
figure(2), mesh(S(1:135),t,V(1:135,:)') 
xlabel('S','FontSize',12), ylabel('t','FontSize',12), 
zlabel('V(x,t)','FontSize',12), hold off
print -r300 -depsc -cmyk '../figures/testHeat_BlackScholes_CrankNicolson'
